{% load i18n %}
{% load static %}
{% load bootstrap3 %}
<html>
  <head>
    <title>  Add New {{ form_prefix }} </title>
  </head>
  <body>
    <h2><i class="glyphicon glyphicon-plus"></i> New {{form_prefix|title}}</h2>
    <div class="container">
      {% if form.errors %}
        <div class="row">
          <div class="alert alert-danger col-sm-offset-2 col-sm-6">
            <p><strong>{% trans "Please correct the errors below: " %}{{ form.field_errors }}</strong></p>
          </div>
        </div>
      {% endif %}
    <div class="col-md-6">
      <form method="post" action="{% url "station_list" %}add/{%filter lower%}{{form_prefix}}{% endfilter %}/"
        class="form-vertical">
        {% csrf_token %}
        {% bootstrap_form form %}
        {% buttons %}
        <p>
          <input type="submit" value="{% trans "Submit Changes" %} &rarr;">
          |<a href="javascript:window.close()">Cancel</a>
        </p>
        {% endbuttons %}
      </form>
    </div>
  </div>
  <script type="text/javascript" src="{% static "admin/js/admin/RelatedObjectLookups.js" %}"></script>
  </body>
</html>
