{% extends "base.html" %}
{% load bootstrap3 %}
{% load i18n %}


{% block title %}{% trans "Password Reset" %}{% endblock %}

{% block content %}
<div class="container">
  <div class="row">
    <div class="col-sm-offset-1 col-sm-10">
      <h1>{% trans "Password Reset" %}</h1>
    </div>
  </div>

  <div class="col-md-6">
      <form action="{% url 'django.contrib.auth.views.password_reset' %}"
            method="post" role="form" class="form-horizontal">
        {% csrf_token %}
        {% bootstrap_form form layout='horizontal' %}

        {% buttons %}
        <div class="col-sm-offset-2 col-sm-10">
          <button type="submit" class="btn btn-primary">{% trans "Reset password" %}</button>
        </div>
        {% endbuttons %}
      </form>
  </div>

</div>
{% endblock %}
