{% extends "base.html" %}
{% load i18n %}

{% block title %}{% trans "Password change complete" %}{% endblock %}

{% block content %}
<div class="container">
  <div class="row">
	<div class="col-md-8">
	  <h1>{% trans "Password changed!" %}</h1>
	  <p>{% trans "Password successfully changed" %}.</p>
    </div>
  </div>
</div>
{% endblock %}
