{% extends "base.html" %}
{% load i18n %}
{% load bootstrap3 %}

{% block title %}{% trans "Login" %}{% endblock %}
{% block content %}
<div class="container">
  <h1>{% trans "Logout" %}</h1>
  <p class="lead">{% trans "You are now logged out" %}.</p>
</div>
{% endblock %}
