{% extends "base.html" %}
{% load bootstrap3 %}
{% load i18n %}

{% block title %}{% trans "Change Password" %}{% endblock %}

{% block content %}
<div class="container">
  <div class="row">
    <div class="col-sm-offset-2 col-sm-10">
      <h1>{% trans "Change Password" %}</h1>
    </div>
  </div>

  <form action="{% url 'password_change' %}" method="post" role="form" class="form-horizontal">
    {% csrf_token %}
    {% bootstrap_form form layout='horizontal' %}

    {% buttons %}
    <div class="col-sm-offset-2 col-sm-10">
      <button type="submit" class="btn btn-primary">{% trans "Change password" %}</button>
    </div>
    {% endbuttons %}
  </form>
</div>
{% endblock %}
