{% extends "base.html" %}
{% load i18n %}
{% load static %}


{% block content %}
  <div class="container">
    <div class="well">
      <h4>{% trans "User profile for" %}
        <a href="{% url "profile_edit" %}"> {{ object }}</a></h4>
        <table>
          <tr>
            <th align="left">{% trans "Username" %}</th>
            <td>{{object.user}} </td>
          </tr>
          <tr>
            <th align="left">{% trans "Name" %}</th>
            <td>{{object.full_name}} </td>
          </tr>
          {% if object.email_is_public %}
            <tr>
              <th align="left">{% trans "Email" %}</th>
              <td>{{object.email}} </td>
            </tr>
          {% endif %}
          <tr>
            <th align="left">{% trans "Address" %}</th>
            <td>{{object.address}} </td>
          </tr>
          <tr>
            <th align="left">{% trans "Organization" %}</th>
            <td>{{object.organization}} </td>
          </tr>
        </table>
        <br>
        <a class="btn btn-xs btn-primary" href="{% url "profile_edit" %}">{% trans "Edit your user profile" %}</a>
        <a class="btn btn-xs btn-primary" href="{% url 'django.contrib.auth.views.password_reset' %}">{% trans "Change your password" %}</a>
        {% if user.is_staff %}
          <a class="btn btn-xs btn-primary" href="{% url "station_list" %}admin/">{% trans "Access the admin panel" %}</a>
          {% endif %}
        </div> <!-- well -->
      </div> <!-- container -->
{% endblock %}
